//****************************************************************************
//FLiNTSample
//@auther Masayuki Daijima
//****************************************************************************

package 
{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.Point;
	
	//FLiNTクラスをインポート
	import org.flintparticles.common.counters.*;
	import org.flintparticles.common.displayObjects.*;
	import org.flintparticles.common.initializers.*;
	import org.flintparticles.twoD.actions.*;
	import org.flintparticles.twoD.emitters.*;
	import org.flintparticles.twoD.initializers.*;
	import org.flintparticles.twoD.renderers.*;
	import org.flintparticles.twoD.zones.*;
	
	//TweenLiteクラスをインポート
	import gs.*;
	import gs.easing.*;
	
	public class FLiNTSample extends Sprite
	{
		private var container		:Sprite;
		private var emitter			:Emitter2D;
		private var renderer		:DisplayObjectRenderer;
		
		private var boundingBox		:BoundingBox;
		private var antiGrav		:MouseAntiGravity;
		private var move			:Move;
		
		private var positionList	:Array = [];
		
		private var mouseMoveStatus	:Boolean;
		private var mouseMoveCount	:int;
		
		public function FLiNTSample():void 
		{
			stage.quality = StageQuality.HIGH;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			initialize();
		}
		
		private function initialize():void 
		{
			container = new Sprite();
			addChild(container);
			
			//パーティクルを放出する元となるオブジェクトを作成
			emitter = new Emitter2D();
			
			//パーティクルの放出パターンとパーティクル数を設定（ここではパーティクルを一回だけ生成するBlastクラスを設定、パーティクル数は750個）
			emitter.counter = new Blast( 750 );
			
			//パーティクルに使用するオブジェクトを設定（ここでは輪郭がぼけたドットを設定）
			emitter.addInitializer( new ImageClass( RadialDot, 2 ) );
			
			//パーティクルの初期位置を設定
			//ここではビットマップデータ（ライブラリ内ZoneBitmapのインスタンス）に基づいてパーティクルが配置されるBitmapDataZoneクラスを使用。
			//ビットマップデータの塗りのある部分にパーティクルが配置されます。
			var bmd = new ZoneBitmap(0, 0);
			emitter.addInitializer( new Position( new BitmapDataZone(bmd, 25, 137) ) );
			
			//パーティクルを表示するレンダラー作成
			renderer = new DisplayObjectRenderer();
			
			//emitterをレンダラーに追加
			renderer.addEmitter( emitter );
			
			//レンダラーを配置
			container.addChild( renderer );
			
			//パーティクルを放出
			emitter.start();
			
			//パーティクルの動く範囲を指定。範囲に達すると逆方向に同じ速度で跳ね返ります。
			boundingBox = new BoundingBox(0, 0, 300, 300);
			
			//マウスの座標からパーティクルを押しのけます。
			antiGrav = new MouseAntiGravity(1, renderer, 1);
			
			//パーティクルを移動させます。
			move = new Move();
			
			//パーティクルの初期位置を配列に格納
			for(var i:uint=0;i<emitter.particles.length;i++){
				var particle = emitter.particles[i];
				var p = {x:particle.x, y:particle.y};
				positionList.push(p);
			}
			
			container.addEventListener(MouseEvent.MOUSE_OVER, mouseOver);
			this.stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMove);
		}
		
		private function mouseOver(event:MouseEvent):void 
		{
			if(!mouseMoveStatus){
				container.mouseEnabled = false;
				this.addEventListener(Event.ENTER_FRAME, onLoop);
				
				//マウスオーバーしたら各アクションを追加
				emitter.addAction( boundingBox );
				emitter.addAction( antiGrav );
				emitter.addAction( move );
				mouseMoveStatus = true;
			}
		}
		
		private function mouseMove(event:MouseEvent):void 
		{
			mouseMoveCount = 0;
		}
		
		private function onLoop(event:Event):void 
		{
			mouseMoveCount++;
			
			//1.5秒間マウスを動かさないでいると、初期位置に戻る
			if(mouseMoveCount>90){
				this.removeEventListener(Event.ENTER_FRAME, onLoop);
				this.stage.removeEventListener(MouseEvent.MOUSE_MOVE, mouseMove);
				
				//各アクションを削除
				emitter.removeAction( boundingBox );
				emitter.removeAction( antiGrav );
				emitter.removeAction( move );
				
				//初期位置に戻す
				for(var i:uint=0;i<emitter.particles.length;i++){
	               	var particle = emitter.particles[i];
	               	particle.velX = 0;
	               	particle.velY = 0;
	               	
	               	if(i == emitter.particles.length-1)TweenLite.to(particle, 1.2, {x:positionList[i].x, y: positionList[i].y, ease:Quad.easeInOut, onComplete:disarm});
					else TweenLite.to(particle, 1.2, {x:positionList[i].x, y: positionList[i].y, ease:Quad.easeInOut});
	            }
			}
		}
		
		private function disarm():void {
            mouseMoveStatus = false;
			container.mouseEnabled = true;
            this.stage.addEventListener(MouseEvent.MOUSE_MOVE, mouseMove);
		}
	}
}