//****************************************************************************
//WOWEngineSample
//@auther Masayuki Daijima
//****************************************************************************

package 
{
	import flash.display.*;
	import flash.events.*;
	import flash.utils.*;
	
	////Papervision3Dクラスをインポート
	import org.papervision3d.cameras.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.render.*;
	import org.papervision3d.scenes.*;
	import org.papervision3d.view.*;
	import org.papervision3d.lights.*;
	import org.papervision3d.materials.utils.*;
	import org.papervision3d.objects.primitives.*;
	import org.papervision3d.materials.shadematerials.*;
	
	//WOWEngineクラスをインポート
	import fr.seraf.wow.primitive.*;
	import fr.seraf.wow.primitive.*;
	import fr.seraf.wow.core.*;
	import fr.seraf.wow.core.data.*;
	
	public class WOWEngineSample extends Sprite 
	{
		private var scene				:Scene3D;
		private var viewport			:Viewport3D;
		private var renderer			:BasicRenderEngine;
		private var camera				:Camera3D;
		
		private var light				:PointLight3D;
		private var ballList			:Array;
		
		private var rootNode			:DisplayObject3D;
		private var cube				:Cube;
		
		private var wow					:WOWEngine;
		private var bound_area			:WBoundArea;
		private var positionList		:Array;
		
		private var ball_cnt			:uint;
		private var rot_cnt				:uint;
		
		public function WOWEngineSample():void 
		{
			stage.quality = StageQuality.HIGH;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			
			initialize();
		}
		
		private function initialize():void 
		{
			ballList = [];
			positionList = [];
			
			init3D();
			initWowEngine();
			setEvent();
			
			rot_cnt = 0;
			this.addEventListener(Event.ENTER_FRAME, onRender);
		}
		
		private function init3D():void 
		{
			//ヴューポートを作成
			viewport = new Viewport3D(stage.stageWidth, stage.stageHeight ,true);
			addChild(viewport);
			
			//レンダラーを作成
			renderer = new BasicRenderEngine();
			
			//シーンを作成
			scene = new Scene3D();
			
			//カメラを設定
			camera = new Camera3D();
			camera.x = 0;
			camera.y = 0;
			camera.z = -1500;
			
			//rootNodeを作成
			rootNode = new DisplayObject3D();
			scene.addChild(rootNode);
			
			//ライトを作成
			light = new PointLight3D();
			
			//Cubeに設定するマテリアルを作成
			var material:WireframeMaterial = new WireframeMaterial(0xFFFFFF, .2);
			material.doubleSided = true;
			
			//Cubeを作成、配置
			cube = new Cube(new MaterialsList({all:material}), 700, 700, 700);
			rootNode.addChild(cube);
		}
		
		private function initWowEngine():void 
		{
			//WOWEngineインスタンスを作成
			wow = new WOWEngine(1);
			
			//衝突時の計算の詳細度（wow.STANDARDは100）
			wow.collisionResponseMode = wow.STANDARD;
			
			//物体にかかる力を設定（Y方向のベクトルを-4に設定）
			wow.addMasslessForce(new WVector(0, -4, 0));
			
			//物理計算をするエリアを設定。引数は(幅、高さ、奥行き)
			bound_area = new WBoundArea(700, 700, 700);
			
			//WBoundsAreaインスタンスの位置
			bound_area.setPosition(0, 0, 0);
			
			//WBoundsAreaの弾性
			bound_area.elasticity = 2;
			
			//WBoundsAreaの摩擦力
			bound_area.friction = 1;
			
			//bound_areaをWOWEngineに設定
			wow.setBoundArea(bound_area);
		}
		
		private function setEvent():void 
		{
			ball_cnt = 0;
			
			//ステージをクリックしたらballを追加
			this.stage.addEventListener(MouseEvent.CLICK, createBall);
		}
		
		private function createBall(event:MouseEvent):void 
		{
			ball_cnt++;
			
			if(ball_cnt<=20){
				//球体に設定するマテリアル
				var material:FlatShadeMaterial = new FlatShadeMaterial(light, 0xFFFFFF*Math.random(), 0x000000);
				
				//表示用の球体を作成
				var ball:Sphere = new Sphere(material, 50);
				rootNode.addChild(ball);
				ballList.push(ball);
				
				
				//物理計算用の球体を作成
				//パラメーターは以下
				//WSphere(X座標 , Y座標, Z座標, 半径, 固定物体にするかどうか（衝突や、力によって影響されるかどうか）, 重さ, 弾性, 摩擦);
				var sphere:WSphere = new WSphere(Math.random()*300-150, Math.random()*300-150, Math.random()*300-150, 50, false, 2, .85, 0);
				
				//初速度を定義
				sphere.velocity = new WVector(Math.random()*100-50, Math.random()*100-50, Math.random()*100-50);
				
				//WOWEngineに登録
				wow.addParticle(sphere);
				
				positionList.push(sphere);
			}
		}
		
		private function onRender(event:Event):void 
		{
			//マウスの動きに合わせて3Dオブジェクトを回転
			rootNode.rotationY -= (this.mouseX-(this.stage.width/2))/50;
			rot_cnt++;
			cube.rotationX = rot_cnt;
			
			//物理計算
			wow.step();
			
			//物理計算の結果を表示に反映
			for(var i:uint=0; i<ballList.length; i++) {
				ballList[i].x = positionList[i].px;
				ballList[i].y = positionList[i].py;
				ballList[i].z = positionList[i].pz;
			}
			
			//物理計算エリアの回転値をcubeの回転値に合わせる
			bound_area.setRotation(rot_cnt, 0, 0);
			
			//3D空間のレンダリング
			renderer.renderScene(scene, camera, viewport);
		}
	}
}