﻿package
{
	import flash.display.Graphics;
	import org.libspark.thread.Thread;
	import org.libspark.thread.utils.IProgress;
	
	/**
	 * このスレッドは、与えられた IProgress インターフェイスのインスタンスが示す進捗状況を、プログレスバーとして描画します。
	 */
	public class ProgressBarThread extends Thread
	{
		/**
		 * @param	progress	進捗状況
		 * @param	graphics	プログレスバーの描画先
		 * @param	width	プログレスバーの最大幅
		 */
		public function ProgressBarThread(progress:IProgress, graphics:Graphics, width:uint)
		{
			_progress = progress;
			_graphics = graphics;
			_width = width;
			// 表示用
			_displayWidth = 0;
		}
		
		private var _progress:IProgress;
		private var _graphics:Graphics;
		private var _width:uint;
		private var _displayWidth:Number;
		
		/**
		 * スレッドの処理は run メソッドをオーバーライドして記述します
		 */	
		override protected function run():void
		{
			// 現在のプログレスバーの幅を算出します
			var w:Number = _width * _progress.percent;
			
			// 表示用の幅を算出します
			// これは、プログレスバーがだんだん伸びるアニメーションを実現するために必要です
			_displayWidth += (w - _displayWidth) / 3;
			
			// プログレスバーを描画します	
			_graphics.clear();
			_graphics.beginFill(0x000000);
			_graphics.drawRect(0, 0, _displayWidth, 350);
			_graphics.endFill();
			
			// 仕事が完了/失敗/キャンセルされていて
			if (_progress.isCompleted || _progress.isFailed || _progress.isCanceled) {
				
				// プログレスバーが完全に伸びきっていたら終了します
				if (_displayWidth >= _width-0.001) {
					return;
				}
			}
			
			// 次もまたこのメソッドを実行します
			next(run);
		}
	}
}